/* Original provenance: */
/*	$OpenBSD: types.h,v 1.12 1997/11/30 18:50:18 millert Exp $	*/
/*	$NetBSD: types.h,v 1.29 1996/11/15 22:48:25 jtc Exp $	*/

#ifndef JOS_INC_TYPES_H
#define JOS_INC_TYPES_H

#ifndef NULL
#define NULL ((void *) 0)
#endif

/* Explicitly-sized versions of integer types */
typedef __signed char int8_t;
typedef unsigned char uint8_t;
typedef short int16_t;
typedef unsigned short uint16_t;
typedef int int32_t;
typedef unsigned int uint32_t;
typedef long long int64_t;
typedef unsigned long long uint64_t;

/* Registers are 32 bits long. */
typedef int32_t register_t;

/* Pointers and addresses are 32 bits long.
 * We use pointer types to represent virtual addresses, uintptr_t to represent
 * the numerical values of virtual addresss, and physaddr_t to represent a
 * physical address. */
typedef int32_t intptr_t;
typedef uint32_t uintptr_t;
typedef uint32_t physaddr_t;

/* Segment indexes are 16 bits long. */
typedef uint16_t segment_t;

/* Page numbers are 32 bits long. */
typedef uint32_t ppn_t;

/* size_t is used for object sizes, and is also 32 bits long. */
typedef uint32_t size_t;
typedef int32_t ssize_t;

/* Efficient min and max operations */ 
#define MIN(_a, _b)	\
	({		\
		typeof(_a) __a = (_a);	\
		typeof(_b) __b = (_b);	\
		__a <= __b ? __a : __b;	\
	})

#define MAX(_a, _b)	\
	({		\
		typeof(_a) __a = (_a);	\
		typeof(_b) __b = (_b);	\
		__a >= __b ? __a : __b;	\
	})

/* 32-bit integer rounding; only works for n = power of two */
#define ROUND32(a, n)		\
	({ uint32_t __n = (n);  (((uint32_t) (a) + __n - 1) & ~(__n - 1)); })
#define ROUNDDOWN32(a, n)	(((uint32_t) (a)) & ~((n) - 1))

/* Static assert, for compile-time assertion checking */
#define static_assert(c)	switch (c) case 0: case (c):

#define offsetof(type, member)  ((size_t)(&((type*)0)->member))

#endif /* !JOS_INC_TYPES_H */
